/*************************************************************************************************

License terms as defined in the file license.txt, which MUST be distributed unmodified with this file.

**************************************************************************************************/

/*
	Create a data feed from AccuWX.
	
	string prefsPrefix - unique code to prepend to all prefs keys, allowing multiple instances.
*/

AccuWeatherFeed = function(prefsPrefix)
{
	var self = this;
	
	// Handy reference to the data retrieval URLs
	
	var dataURL = "http://touchthesky.accuweather.com/adcbin/touchthesky/weather_data.asp";
	var mapsURL = "http://maps.google.com/maps?hl=en&z=14";

	// Current local time needs updated every minute 

	var localTimeUpdateTimer = null;
	
	// The Yahoo! YUI AJAX transaction object - we re-use it so we can test if it's still running or not 
	
	var transaction = null;

	// The feed's current status 

	self.loaded = false;
	self.needsUpdate = true;
	
	// Initial objects to store the various types of data 
	
	self.location = {};
	self.units = {};
	self.current = {};
	self.forecast = {};
	self.satellite = {};
	self.planets = {};
	self.air = {};
	
	// Preference objects - current AccuWX location code, metric/US units, windspeed to be converted to km/h, use RealFeel instead
	// of actual measurement
	
	self.locationCode = new Preference(prefsPrefix + "_location", "EUR|NO|NO011|OSLO|");
	self.locationCode.onChange = function()	{ self.needsUpdate = true; }
	self.locationCode.load();
	
	self.basicUnits = new Preference(prefsPrefix + "_units", "metric");
	self.basicUnits.onChange = function() { self.needsUpdate = true; }
	self.basicUnits.load();
	
	self.windSpeedKmH = new Preference(prefsPrefix + "_kmh", true, "bool");
	self.windSpeedKmH.onChange = function() { self.needsUpdate = true; }
	self.windSpeedKmH.load();
	
	self.realFeel = new Preference(prefsPrefix + "_realfeel", false, "bool");
	self.realFeel.onChange = function() { self.needsUpdate = true; }
	self.realFeel.load();

	/* Start reloading the data from the AJAX feed */
	
	self.reload = function(callback)
	{
		// Set the location name to Unknown - so even if reload fails, it doesn't look like live data 
		
		self.location.shortName = "Unknown";
		self.location.longName = self.location.shortName;

		// Safety check - if not running as a widget, don't do anything 

		if (typeof widget != "undefined")
		{
			// Set feed status to 'not loaded' 
			
			self.loaded = false;

			// Push HTTP params onto an array, we can join them into a single string later. This is clearer. 
			
			var httpParams = [];
			httpParams.push("location=" + self.locationCode.get());
			httpParams.push("metric=" + ((self.basicUnits.get() == "metric") ? "1" : "0"));
			
			// Use YUI - set our own handlers for success and failure, pass the caller's handlers as an argument 

			var yahooCallback = { success: reloadSuccess, failure: reloadFailure, argument: callback };	
			
			// If earlier transaction is in progress, abort it 
			
			if (transaction && YAHOO.util.Connect.isCallInProgress(transaction))
			{
				YAHOO.util.Connect.abort(transaction);
			}
			
			// Ensure data isn't cached, or our local time will get out of sync, then make the call 
			
			YAHOO.util.Connect.initHeader("Cache-Control", "no-cache");
			transaction = YAHOO.util.Connect.asyncRequest('GET', dataURL+"?"+httpParams.join("&"), yahooCallback);
		}
	}
	
	/* Reload failed, report it to the caller */

	var reloadFailure = function(response)
	{
		// Retrieve caller's callback functions from the YUI argument 
		
		var callback = response.argument;
		
		if (callback.failure) callback.failure(response.status, response.statusText);
	}
	
	/* Reload successful, report it to the caller */
		
	var reloadSuccess = function(response)
	{
		// Retrieve caller's callback functions from the YUI argument 
		
		var callback = response.argument;
		
		var xmlRoot = response.responseXML;
		
		// Helper functions for parsing pollen/mould counts and planetary rise/set times 
		
		var parseParticleLevel = function(code)
		{
			switch (code.charAt(0))
			{
				case "N": return("No"); break;
				case "L": return("Low"); break;
				case "M": return("Moderate"); break;
				case "H": return("High"); break;
				case "V": return("Very High"); break;
			}
		}
		
		var parsePlanet = function(parent, tag)
		{
			var planet = parent.getElementsByTagName(tag)[0];
			
			return { rise: planet.getAttribute("rise"), set: planet.getAttribute("set") };
		}

		// Check for AccuWX failure code 
			
		if (xmlRoot.getElementsByTagName("failure").length > 0)
		{
			// If a failure callback was given, call it with a suitable error message 
			if (callback.failure)
			{
				var errMessage = xmlRoot.getElementsByTagName("failure")[0].firstChild.nodeValue;
				
				if (errMessage.indexOf("ocation") > 0 && errMessage.indexOf("not exist") > 0)
				{
					callback.failure(response.status, "could not find data for this location. Please try another.");
				}
				else
				{
					callback.failure(response.status, errMessage);
				}
			}
		}
		else
		{
			// Traverse the XML DOM for data 
		
			var useRealFeel = self.realFeel.get();
			
			self.dateLoaded = new Date().formatTemplate("{DD} {dd} {MM}");
			self.timeLoaded = new Date().formatTemplate("{hh}:{mm}");
			var localTag = xmlRoot.getElementsByTagName("local")[0];
			self.location.shortName = localTag.getAttribute("city");
			self.location.longName = self.location.shortName + ", " + localTag.getAttribute("state");
			self.location.time = localTag.getAttribute("time");
			self.location.latitude = localTag.getAttribute("lat");
			self.location.longitude = localTag.getAttribute("lon");
			self.location.mapLink = mapsURL + "&ll=" + self.location.latitude + "," + self.location.longitude;

			self.satellite.img = xmlRoot.getElementsByTagName("radar")[0].firstChild.nodeValue;
			self.satellite.link = xmlRoot.getElementsByTagName("radurl")[0].firstChild.nodeValue;

			var unitsTag = xmlRoot.getElementsByTagName("units")[0];
			self.units.temperature = String.fromCharCode(176) + unitsTag.getAttribute("temp");
			self.units.distance = unitsTag.getAttribute("dist");
			self.units.speed = unitsTag.getAttribute("speed");
			var convertToKmH = (self.basicUnits.get() == "metric" && self.units.speed == "m/s" && self.windSpeedKmH.get() == true);
			if (convertToKmH) self.units.speed = "km/h";
			self.units.pressure = unitsTag.getAttribute("pres");
			self.units.precipitation = unitsTag.getAttribute("prec");

			var currentTag = xmlRoot.getElementsByTagName("currentconditions")[0];
			var currentStats = currentTag.getElementsByTagName("wxc")[0];
			self.current.link = currentTag.getElementsByTagName("url")[0].firstChild.nodeValue;
			if (useRealFeel)
				self.current.temperature = currentStats.getAttribute("rft");
			else
				self.current.temperature = currentStats.getAttribute("temp");
			self.current.text = currentStats.getAttribute("text");
			self.current.icon = currentStats.getAttribute("icon");

			self.current.windSpeed = currentStats.getAttribute("wspd");
			if (convertToKmH)
				self.current.windSpeed = Math.round(self.current.windSpeed * 60 * 60 / 1000);
			self.current.windGusts = currentStats.getAttribute("wgus");
			if (convertToKmH)
				self.current.windGusts = Math.round(self.current.windGusts * 60 * 60 / 1000);
			self.current.windDirection = currentStats.getAttribute("wdir");
			if (self.current.windDirection == "CLM")
				self.current.windDirection = "Calm";

			self.current.pressure = currentStats.getAttribute("pres");
			self.current.pressureTrend = currentStats.getAttribute("ptend");
			
			self.current.precipitation = currentStats.getAttribute("prec");
			self.current.visibility = currentStats.getAttribute("vis");
			self.current.humidity = currentStats.getAttribute("hum").replace(new RegExp("\\.[0-9]+%"),"%");
			self.current.uvIndex = currentStats.getAttribute("uvt");

			var airTag = xmlRoot.getElementsByTagName("airandpollen")[0];

			self.air.grassPollen = parseParticleLevel(airTag.getAttribute("grass"));
			self.air.treePollen = parseParticleLevel(airTag.getAttribute("tree"));
			self.air.weedPollen = parseParticleLevel(airTag.getAttribute("weed"));
			self.air.mold = parseParticleLevel(airTag.getAttribute("mold"));

			self.air.quality = "";
			var qualityCode = airTag.getAttribute("airq");
			switch (qualityCode.charAt(0))
			{
				case "G": self.air.quality = "Good"; break;
				case "M": self.air.quality = "Moderate"; break;
				case "U": self.air.quality = "Unhealthy"; break;
				case "V": self.air.quality = "Very Unhealthy"; break;
				case "H": self.air.quality = "Hazardous"; break;
			}
			
			var planetsTag = xmlRoot.getElementsByTagName("planets")[0];
			var moonPhaseTag = xmlRoot.getElementsByTagName("phase")[0];
			
			self.planets.sun = parsePlanet(planetsTag, "sun");
			self.planets.moon = parsePlanet(planetsTag, "moon");
			self.planets.mercury = parsePlanet(planetsTag, "mercury");
			self.planets.venus = parsePlanet(planetsTag, "venus");
			self.planets.mars = parsePlanet(planetsTag, "mars");
			self.planets.jupiter = parsePlanet(planetsTag, "jupiter");
			self.planets.saturn = parsePlanet(planetsTag, "saturn");
			self.planets.uranus = parsePlanet(planetsTag, "uranus");
			self.planets.neptune = parsePlanet(planetsTag, "neptune");
			self.planets.pluto = parsePlanet(planetsTag, "pluto");

			self.planets.moon.phaseText = moonPhaseTag.getAttribute("text");
			self.planets.moon.phaseIcon = moonPhaseTag.firstChild.nodeValue;
			
			var forecastTag = xmlRoot.getElementsByTagName("forecast")[0];
			self.forecast.link = forecastTag.getElementsByTagName("url")[0].firstChild.nodeValue;
			
			self.forecast.day = [];
			
			var dayTags = forecastTag.getElementsByTagName("day");

			var moonTags = xmlRoot.getElementsByTagName("moon")[1].getElementsByTagName("phase");
			for (var i=0; i<dayTags.length; i++)
			{
				var dayNumber = dayTags[i].getAttribute("number")-1;
				
				self.forecast.day[dayNumber] = {};
				self.forecast.day[dayNumber].dayTime = {};
				self.forecast.day[dayNumber].nightTime = {};

				var urlTag = dayTags[i].getElementsByTagName("url")[0];
				
				var dateSplit = dayTags[i].getAttribute("date").split("/");
				self.forecast.day[dayNumber].date = new Date(dateSplit[2],dateSplit[0]-1,dateSplit[1]);
				self.forecast.day[dayNumber].weekday = dayTags[i].getAttribute("wday");
				self.forecast.day[dayNumber].link = urlTag.firstChild.nodeValue;
				
				var sunTag = dayTags[i].getElementsByTagName("sun")[0];
				var phaseDate = dayTags[i].getAttribute("date");
				self.forecast.day[dayNumber].sunRise = sunTag.getAttribute("rise");
				self.forecast.day[dayNumber].sunSet = sunTag.getAttribute("set");
				self.forecast.day[dayNumber].moonText = "";
				self.forecast.day[dayNumber].moonIcon = "10";
				for (var j=0; j<moonTags.length; j++)
				{
					if (moonTags[j].getAttribute("date") == phaseDate)
					{
						self.forecast.day[dayNumber].moonText = moonTags[j].getAttribute("text");
						self.forecast.day[dayNumber].moonIcon = moonTags[j].firstChild.nodeValue;
					}
				}
				
				var dayTimeTag = dayTags[i].getElementsByTagName("daytime")[0];
				var dayTimeStats = dayTimeTag.getElementsByTagName("wxf")[0];
				var nightTimeTag = dayTags[i].getElementsByTagName("nighttime")[0];
				var nightTimeStats = nightTimeTag.getElementsByTagName("wxf")[0];

				self.forecast.day[dayNumber].dayTime.text = dayTimeTag.getElementsByTagName("txtshort")[0].firstChild.nodeValue;
				if (useRealFeel)
				{
					self.forecast.day[dayNumber].dayTime.highTemperature = dayTimeStats.getAttribute("rfh");
					self.forecast.day[dayNumber].dayTime.lowTemperature = dayTimeStats.getAttribute("rfl");
				}
				else
				{
					self.forecast.day[dayNumber].dayTime.highTemperature = dayTimeStats.getAttribute("htmp");
					self.forecast.day[dayNumber].dayTime.lowTemperature = dayTimeStats.getAttribute("ltmp");
				}
				self.forecast.day[dayNumber].dayTime.icon = dayTimeStats.getAttribute("icon");
				self.forecast.day[dayNumber].dayTime.windSpeed = dayTimeStats.getAttribute("wspd");
				self.forecast.day[dayNumber].dayTime.windDirection = dayTimeStats.getAttribute("wdir");
				self.forecast.day[dayNumber].dayTime.windGusts = dayTimeStats.getAttribute("wgus");
				self.forecast.day[dayNumber].dayTime.rain = dayTimeStats.getAttribute("rain");
				self.forecast.day[dayNumber].dayTime.snow = dayTimeStats.getAttribute("snow");

				self.forecast.day[dayNumber].nightTime.text = nightTimeTag.getElementsByTagName("txtshort")[0].firstChild.nodeValue;
				if (useRealFeel)
				{
					self.forecast.day[dayNumber].nightTime.highTemperature = nightTimeStats.getAttribute("rfh");
					self.forecast.day[dayNumber].nightTime.lowTemperature = nightTimeStats.getAttribute("rfl");
				}
				else
				{
					self.forecast.day[dayNumber].nightTime.highTemperature = nightTimeStats.getAttribute("htmp");
					self.forecast.day[dayNumber].nightTime.lowTemperature = nightTimeStats.getAttribute("ltmp");
				}
				self.forecast.day[dayNumber].nightTime.icon = nightTimeStats.getAttribute("icon");
				self.forecast.day[dayNumber].nightTime.windSpeed = nightTimeStats.getAttribute("wspd");
				self.forecast.day[dayNumber].nightTime.windDirection = nightTimeStats.getAttribute("wdir");
				self.forecast.day[dayNumber].nightTime.windGusts = nightTimeStats.getAttribute("wgus");
				self.forecast.day[dayNumber].nightTime.rain = nightTimeStats.getAttribute("rain");
				self.forecast.day[dayNumber].nightTime.snow = nightTimeStats.getAttribute("snow");

			}

			self.loaded = true;
			self.needsUpdate = false;
		}

		// If a success callback was defined, call it now 

		if (callback.success) callback.success();
	}
	
	/* Every minute, update the local time data. Create a timer to do so. */
	
	var updateLocalTime = function()
	{
		if (self.loaded)
		{
			var hours = parseInt(self.location.time.substring(0,2));
			var mins = parseInt(self.location.time.substring(3,5));
			mins = mins + 1;
			if (mins >= 60)
			{
				mins = mins - 60;
				hours = hours + 1;
			}
			if (hours >= 24)
			{
				hours = hours - 24;
			}

			hours = hours.toString();
			if (hours.length == 1) hours = "0" + hours;
			mins = mins.toString();
			if (mins.length == 1) mins = "0" + mins;

			self.location.time = hours + ":" + mins;
		}
	}

	localTimeUpdateTimer = window.setInterval(updateLocalTime, 60*1000);

}

/*********************/

/*
	A class to search for city name matches in AccuWX's list.
*/

LocationFinder = function()
{
	var self = this;
	
	var searchURL = "http://touchthesky.accuweather.com/adcbin/touchthesky/locate_city.asp";

	self.results = [];
	
	// As above, use single AJAX transaction object 
	
	var transaction = null;
	
	// Do the search 
	
	self.search = function(searchTerm, callback)
	{
		// Safety check - if not running as a widget, do nothing 
	
		if (typeof widget != "undefined")
		{
			// As above, push HTTP params onto an array, to join() later 
			
			var httpParams = [];
			httpParams.push("location="+searchTerm);
			
			// Pass caller's callbacks as an argument and use our own 
			
			var yahooCallback = { success: searchSuccess, failure: searchFailure, argument: callback };

			// Abort any existing transaction 

			if (transaction && YAHOO.util.Connect.isCallInProgress(transaction))
			{
				YAHOO.util.Connect.abort(transaction);
			}

			transaction = YAHOO.util.Connect.asyncRequest('GET', searchURL+"?"+httpParams.join("&"), yahooCallback);
		}
	}

	/* Success - populate results, and call the caller's success callback */

	var searchSuccess = function(response)
	{
		var callback = response.argument;
		var xmlRoot = response.responseXML;
		var locationElems = xmlRoot.getElementsByTagName("location");
		
		// Clear results list, and populate with data objects 
		
		self.results = [];
		for (var i=0; i<locationElems.length; i++)
		{				
			self.results.push({city: locationElems[i].getAttribute("city"), state: locationElems[i].getAttribute("state"), locationCode: locationElems[i].getAttribute("location")});
		}
		
		if (callback.success) callback.success();
	}
	
	/* Failed - report to the caller */
	
	var searchFailure = function(response, callback)
	{
		var callback = response.argument;
		
		if (callback.failure) callback.failure(response.status, response.statusText);
	}

	
}