/*************************************************************************************************

License terms as defined in the file license.txt, which MUST be distributed unmodified with this file.

**************************************************************************************************/

/*
	Implemented as an instantiable class - theoretically, you could have multiple instances of the widget.
	Either way, this object encapsulates everything - no namespace pollution.
	All member variables are private.
	
	All the library classes (DrawerManager, AccuWeatherFeed, etc.) are independent of the rest of the widget, of
	the DOM, and of each other - allowing multiple instances. I've been (relatively!) strict, they maintain their
	own state and data.
	The touchtheSky object's job is to manage the interactions between them, allowing a more pluggable architecture.
*/
   
touchtheSky = function()
{
	var homeURL = "http://home.accuweather.com/index.asp?partner=touchthesky";
	var updateURL = "http://touchthe.net/taxonomy/term/7%2C13/1/feed";
	
	// Local mode flag for testing - update checks won't work on my local network if I also access internet data 
	var localMode = false;
	
	// Timer for the loading throbber 
	var loadingAnim = null;
	var loadingAnimFrame = 0;
	
	// Timer to reload data every half-hour 
	var reloadDataTimer = null;
	
	// General preference settings
	var mode = new Preference("mode", "bar");
	var skin = new Preference("skin", "plain");
	var opacity = new Preference("opacity", "1.0", "float");
	
	// Warning pop-up boxes - only alertMessage is currently used 
	var infoMessage = new AlertMessage("infomessage","info","shared/img/biginfo.png");
	var alertMessage = new AlertMessage("alertmessage","alert","shared/img/bigalert.png");
	var warningMessage = new AlertMessage("warningmessage","warning","shared/img/bigwarning.png");
	
	// Drawer manager which maintains order amongst the drawers, including remembering last one used on startup 
	var drawerMgr = new DrawerManager("drawer", "about");
	
	// Create all the drawers, and add them to the default drawer manager 
	var satelliteDrawer = drawerMgr.addDrawer(new Drawer("satellite","satellite_drawer","satellite_button"));
	var nowDrawer = drawerMgr.addDrawer(new Drawer("now","now_drawer","now_button"));
	var forecastDrawer = drawerMgr.addDrawer(new Drawer("forecast","forecast_drawer","forecast_button"));
	var aboutDrawer = drawerMgr.addDrawer(new Drawer("about","about_drawer","about_button"));
	var configDrawer = drawerMgr.addDrawer(new Drawer("config","config_drawer","config_button"));
	
	// Create a weather data source and location search source - pluggable, can be switched from AccuWeather
	// later without affecting any other code
	var weatherFeed = new AccuWeatherFeed("accuweather")
	var locationFinder = new LocationFinder();
	
	// Create an object to check for updates to the software 
	var updater = new NewsChecker(updateURL, "updates");

	// Create the two main faces, the mini-mode version and full-length bar 
	var miniFace = new Face("mini");
	var barFace = new Face("bar");
	
	// Two of the drawers use sliders to page between data sets
	var nowSlider = new Slider("now_slider", 260);
	var forecastSlider = new Slider("forecast_slider", 260);

/*******************/

	// Called when DOM has been initialised - set up everything DOM-dependent 
	var init = function()
	{
		infoMessage.init();
		alertMessage.init();
		warningMessage.init();
		
		miniFace.init();
		barFace.init();
		
		drawerMgr.init();
		
		attachFaceButtonHandlers();
		
		// load preferences 
		mode.load();
		skin.load();
		opacity.load();
		
		// redraw non-weather drawers, so they match saved configuration 
		configDrawer.redraw();
		aboutDrawer.redraw();
		
		attachConfigChangeHandlers();
		
		nowSlider.init();
		forecastSlider.init();

		attachTabHandlers("now_tabs", nowSlider);
		attachTabHandlers("forecast_tabs", forecastSlider);
		
		// everything is ready, display the widget and starting loading the weather data 
		
		document.body.style.display = 'block';
		
		reloadData();
		
		// Uncomment the following line to force the Last Updated value to Jan 1, 1970 
		//updater.resetLastUpdate();

		// If the auto-update is enabled, check the feed for news items in 2 seconds
		// call updateSuccess if able to retrieve data, updateFailure otherwise
		if (updater.enabled.get() == true)
		{
			updater.delayedCheck( { success: updateSuccess, failure: updateFailure }, 2);
		}
		
	}
	
	// When DOM is loaded, call init method 
	document.addEventListener("load", init, false);

/*******************/

	/* When value of mode has changed, switch to 'bar' or 'mini' face (closing open drawer if necessary) */
	
	mode.onChange = function()
	{
		var newFace = mode.get();
		if (newFace == "bar")
		{
			miniFace.close(true);
			barFace.open(true);
		}
		else
		{
			drawerMgr.closeActive(true);
			
			// do this fast for now (no animation) - Opera on Windows doesn't like opacity of translucent PNGs 
			barFace.close(true);
			miniFace.open(true);
		}
	}
		
	// opacity disabled due to incorrect handling of opacity with translucency in current versions of Opera on Windows (perhaps others)
	//opacity.onChange = function()
	//{
	//	document.body.style.opacity = opacity.get();
	//}
	//opacity.load();
	
	// When value of skin has changed, change <body> class to new skin 
	skin.onChange = function() { document.body.className = skin.get(); }

/*******************/
	
	/* Redraw methods for all the drawers - these override the default methods for Drawer objects
	   The first few redraw themselves according to the data loaded from the weatherFeed */
	
	satelliteDrawer.redraw = function()
	{
		document.getElementById("satellite_image").src = weatherFeed.satellite.img;
		document.getElementById("satellite_link").href = weatherFeed.satellite.link;
		document.getElementById("satellite_lastupdated").removeChildren().appendChild(document.createTextNode(weatherFeed.dateLoaded + " @ " + weatherFeed.timeLoaded));
	}

	nowDrawer.redraw = function()
	{
		// A little helper function to aid the text-swapping 
		
		var replaceText = function(elemID, newText)
		{
			document.getElementById(elemID).removeChildren().appendChild(document.createTextNode(newText));
		}
		
		// Replace the attractive full-size weather icon 
		
		var icon = document.getElementById("now_icon");
		icon.src = "weathericons/" + weatherFeed.current.icon + ".png";
		icon.alt = weatherFeed.current.text;

		// Replace the basic data
		
		replaceText("now_location", weatherFeed.location.shortName);
		replaceText("now_desc", weatherFeed.current.text);
		replaceText("now_temp", weatherFeed.current.temperature + weatherFeed.units.temperature);
		replaceText("now_wind", "Wind " + weatherFeed.current.windDirection + " at " + weatherFeed.current.windSpeed + weatherFeed.units.speed);
		replaceText("now_gust", "Gusts to " + weatherFeed.current.windGusts + weatherFeed.units.speed);
		
		document.getElementById("now_link").href = weatherFeed.current.link;
		replaceText("now_time", "Local time " + weatherFeed.location.time);
		
		// Replace the atmospheric data 
		
		replaceText("now_quality", weatherFeed.air.quality + " air quality");
		replaceText("now_grass", weatherFeed.air.grassPollen + " grass pollen");
		
		replaceText("now_uvindex", weatherFeed.current.uvIndex + " UV");
		replaceText("now_humidity", weatherFeed.current.humidity + " humidity");
		
		var moonicon = document.getElementById("now_moonicon");
		moonicon.src = "moonicons/" + weatherFeed.planets.moon.phaseIcon + ".png";
		moonicon.alt = weatherFeed.planets.moon.phaseText;
		replaceText("now_moonphase", weatherFeed.planets.moon.phaseText + " moon");
		
		// Let user know when data was last refreshed 
		
		replaceText("now_lastupdated", weatherFeed.dateLoaded + " @ " + weatherFeed.timeLoaded);
	}
	
	/* The 'now' drawer also needs to update its Local Time display once a minute.
	   Give it a method to do so, and set an interval timer. */
	
	nowDrawer.redrawLocalTime = function()
	{
		if (weatherFeed.loaded)
		{
			var localTime = document.getElementById("now_time");
			localTime.removeChildren().appendChild(document.createTextNode("Local time " + weatherFeed.location.time));
		}
	}
	
	nowDrawer.localTimeRefreshTimer = window.setInterval(nowDrawer.redrawLocalTime, 60*1000);

	forecastDrawer.redraw = function()
	{
		var forecastList = document.getElementById("forecast_slider");
		var forecastSummary = document.getElementById("forecast_summary");
		var forecastSummaryHeader = forecastSummary.getElementsByTagName("tr")[0];
		
		// Clear out all the various forecast pages, remembering to re-add the headings and the summary page 
		
		forecastList.removeChildren();
		forecastSummary.removeChildren();
		forecastSummary.appendChild(forecastSummaryHeader);
		forecastList.appendChild(forecastSummary);

		// Update the link to the full forecast page at AccuWX 
		
		document.getElementById("forecast_link").href = weatherFeed.forecast.link;
		var tabs = document.getElementById("forecast_tabs").getElementsByTagName("button");
		
		// For each forecast day... 
		for (var i=0; i<5; i++)
		{
			// ...if this day exists in the weather feed (prevents unintentional overruns)... 
			if (weatherFeed.forecast.day[i])
			{
				// ...and its not today ...
				if (i != 0)
				{
					// ...rename the relevant tab on the forecast page to the day of the week 
					tabs[i+1].removeChildren().appendChild(document.createTextNode(weatherFeed.forecast.day[i].date.formatTemplate("{DD}")));
				}
				// make sure the tab can now be clicked 
				tabs[i+1].disabled = false;

				// add a new forecast page for this day 
				var newItem = new ForecastItem(weatherFeed.forecast.day[i], weatherFeed.units);
				newItem.appendTo(forecastList);
				
				// and add a summary row to the summary page 
				var newSummaryItem = new ForecastSummaryItem(weatherFeed.forecast.day[i], weatherFeed.units);
				newSummaryItem.appendTo(forecastSummary);
			}
		}
		
	}
	
	/* These two drawers only need to redraw the configuration settings when first loaded */

	aboutDrawer.redraw = function()
	{
		// check the box if auto-update is enabled 
		
		document.getElementById("update_check").checked = updater.enabled.get();
	}

	configDrawer.redraw = function()
	{
		var locationInput = document.getElementById("location_input");
		locationInput.value = locationInput.defaultValue;

		// For each radio button in each set, if that button corresponds to the current config setting, select it 

		var unitsRadios = document.getElementsByName("units");
		var currentUnits = weatherFeed.basicUnits.get();
		for (var i=0; i<unitsRadios.length; i++)
		{
			if (currentUnits == unitsRadios[i].value)
				unitsRadios[i].checked = true;
		}

		var speedRadios = document.getElementsByName("units_speed");
		var speedUnits = (weatherFeed.windSpeedKmH.get() ? "km/h" : "m/s");
		for (var i=0; i<speedRadios.length; i++)
		{
			if (speedUnits == speedRadios[i].value)
				speedRadios[i].checked = true;
			speedRadios[i].disabled = (currentUnits != "metric");
		}

		var tempRadios = document.getElementsByName("temperature");
		var tempMeasure = (weatherFeed.realFeel.get() ? "realfeel" : "actual");
		for (var i=0; i<tempRadios.length; i++)
		{
			if (tempMeasure == tempRadios[i].value)
				tempRadios[i].checked = true;
		}

		// Select current appearance settings 

		document.getElementById("skin").value = skin.get();
		document.getElementById("fast").checked = drawerMgr.fastSwitch.get();
	}
	
	/* Before the drawer manager closes a drawer - if it was the config drawer and changes require data to be reloaded, do it.
	   This overrides a default method in DrawerManager */

	drawerMgr.beforeClose = function()
	{
		if (drawerMgr.isActive(configDrawer))
		{
			if (weatherFeed.needsUpdate)
			{
				reloadData();
			}
			showPendingWarning(false);
			document.getElementById("location_select").selectedIndex = -1; // work around bug in 9.02 
		}
	}

/*******************/

	/* When data is updated, the drawers aren't the only things needing updated. Update the faces too. */

	miniFace.redraw = function()
	{
		var icon = document.getElementById("mini_icon");
		icon.src = "weathericons/" + weatherFeed.current.icon + ".png";
		icon.alt = weatherFeed.current.text;
		
		var label = document.getElementById("mini_label");
		label.removeChildren().appendChild(document.createTextNode(weatherFeed.location.shortName));
	
		var summary = document.getElementById("mini_summary");
		summary.removeChildren().appendChild(document.createTextNode(weatherFeed.current.text));
		
		var temp = document.getElementById("mini_temp");
		temp.removeChildren().appendChild(document.createTextNode(weatherFeed.current.temperature + weatherFeed.units.temperature));
		
		var wind = document.getElementById("mini_wind");
		wind.removeChildren().appendChild(document.createTextNode(weatherFeed.current.windSpeed + weatherFeed.units.speed + " " + weatherFeed.current.windDirection));
	}
		
	barFace.redraw = function()
	{
		setStatusLabel(weatherFeed.location.shortName, "weathericons/" + weatherFeed.current.icon + ".png");
	}

	/* Other setup for the faces - attach handlers for the non-drawer buttons */
	
	var attachFaceButtonHandlers = function()
	{
		document.getElementById("maximise_button").addEventListener("mousedown", function() { mode.set("bar"); }, false);
		document.getElementById("minimise_button").addEventListener("mousedown", function() { mode.set("mini"); }, false);
			
		document.getElementById("close_button").addEventListener("mousedown", function() { window.close(); }, false);
		document.getElementById("miniclose_button").addEventListener("mousedown", function() { window.close(); }, false);
	}

/*******************/

	/* Auto-update check was successful - see if there's any unread news to display */
	
	var updateSuccess = function ()
	{
		if (updater.hasUnreadItems)
		{
			// If current drawer isn't About, close it and open About instead 
			
			if (!drawerMgr.isActive(aboutDrawer))
			{
				drawerMgr.closeActive();
				drawerMgr.openActive(aboutDrawer);
			}
			
			// Only display the most recent item 
			
			var lastItem = updater.items[updater.items.length - 1];

			// Insert the title, link, description, date, and download link 

			document.getElementById("update_title").firstChild.nodeValue = lastItem.title;
			document.getElementById("update_link").src = lastItem.link;
			document.getElementById("update_text").innerHTML = lastItem.description;
			document.getElementById("update_date").firstChild.nodeValue = lastItem.shortDate;
			if (lastItem.download)
			{
				document.getElementById("update_download").src = lastItem.download;
			}
			
			// Now hide the 'No Updates Found' message and show the news item 
			document.getElementById("update_none").style.display = "none";
			document.getElementById("update_message").style.display = "block";
		}
		else
		{
			// Hide the news item area and show the 'No Updates Found' message 
			document.getElementById("update_message").style.display = "none";
			document.getElementById("update_none").style.display = "block";
		}
	}
	
	/* Auto-update failed - pop-up a dialog box */
	
	var updateFailure = function(statusCode, statusText)
	{
		if (statusCode == 408)
			statusText = "timed out connecting to update server";
		else
			statusText = "failed to check for updates";
		
		alertMessage.show("Error - " + statusText);
	}

/*******************/

	/* Time to reload the weather data */

	var reloadData = function()
	{
		if (localMode) return;
		
		// Reset the timer, just in case... 
		if (reloadDataTimer)
		{
			window.clearTimeout(reloadDataTimer);
			reloadDataTimer = null;
		}
		
		// ...and show the loading indicator 
		showLoadingIndicator(true);
		
		// Ask the weather feed to update itself, call reloadDataSuccess on success, otherwise reloadDataFailure 
		weatherFeed.reload( { success: reloadDataSuccess, failure: reloadDataFailure } );
	}

	/* Data update was successful, redraw the weather-display drawers and faces */

	var reloadDataSuccess = function()
	{
		satelliteDrawer.redraw();
		nowDrawer.redraw();
		forecastDrawer.redraw();
		miniFace.redraw();
		barFace.redraw();
		
		reloadDataFinish();
	}
	
	/* Data update failed, pop-up an alert message */

	var reloadDataFailure = function(statusCode, statusMessage)
	{
		if (statusCode == 404) statusMessage = "data not found";
		else if (statusCode > 400 || statusCode < 100) statusMessage = "could not connect to server";
		
		setStatusLabel("Error", "weathericons/swa.png");
		alertMessage.show("Error - " + statusMessage);
		reloadDataFinish();
	}
	
	/* Tidy up, switch off loading throbber and wait another 30 mins before trying again */

	var reloadDataFinish = function()
	{
		showLoadingIndicator(false);
		reloadDataTimer = window.setTimeout(reloadData,30*60*1000); // 30 mins * 60 secs * 1000ms
	}

/*******************/

	/* Ask location finder to search for the city entered in search form */

	var searchLocations = function()
	{
		if (localMode) return;
	
		var locationInput = document.getElementById("location_input");
		var locationSubmit = document.getElementById("location_submit");
		var locationSelect = document.getElementById("location_select");

		// Disable the search form controls to avoid multiple submissions, and clear any old results 

		locationInput.disabled = true;
		locationSubmit.disabled = true;
		locationSelect.selectedIndex = -1;
		locationSelect.removeChildren();
		locationSelect.disabled = true;

		// Show loading throbber for duration of check 

		showLoadingIndicator(true);

		// Ask location finder to do its stuff, with callback functions for success/failure 

		locationFinder.search(locationInput.value, { success: searchLocationsSuccess, failure: searchLocationsFailure } );
	}

	/* Search was successful - get location codes from search results, and add them as options to the list */

	var searchLocationsSuccess = function()
	{
		var locationSelect = document.getElementById("location_select");
		
		// Get current location, so we can pre-select it in the list 
		var currentLocationCode = weatherFeed.locationCode.get();
		
		if (locationFinder.results.length == 0)
		{
			var noLocationsOption = document.createElement("option");
			noLocationsOption.appendChild(document.createTextNode("-- No locations found. Please try another name. --"));
			noLocationsOption.value = "";
			locationSelect.appendChild(noLocationsOption);
		}
		else
		{
			for (var i=0; i<locationFinder.results.length; i++)
			{
				var thisLocation = locationFinder.results[i];
			
				var newLocationOption = document.createElement("option");
				newLocationOption.appendChild(document.createTextNode(thisLocation.city +", " + thisLocation.state));
				newLocationOption.value = thisLocation.locationCode;
				locationSelect.appendChild(newLocationOption);
			
				if (currentLocationCode == thisLocation.location) locationSelect.selectedIndex = i;
			}
		}
			
		searchLocationsFinish();
		
	}
	
	/* Search failed - explain why in a dialog box, and ensure the bar label displays an error icon */
	
	var searchLocationsFailure = function(statusCode, statusMessage)
	{
		if (statusCode == 404)
		{
			statusMessage = "data not found";
		}
		else if (statusCode > 400 || statusCode < 100)
		{
			statusMessage = "could not connect to server";
		}
		
		setStatusLabel("Error", "weathericons/swa.png");
		alertMessage.show("Error - " + statusMessage);
		
		searchLocationsFinish();
	}
	
	/* Tidy up, switch off loading throbber, and re-enable form controls */
	
	var searchLocationsFinish = function()
	{
		document.getElementById("location_input").disabled = false;
		document.getElementById("location_submit").disabled = false;
		document.getElementById("location_select").disabled = false;
				
		showLoadingIndicator(false);
	}

/*******************/

	/* These may end up in a shared library at a future date */

	/* Update throbber animation frame by switching CSS background-position */
	
	var updateLoadingIndicator = function()
	{
		var statusBox = document.getElementById("status");
		statusBox.style.backgroundPosition = "0px -" + (loadingAnimFrame*16) + "px";
		loadingAnimFrame += 1;
		if (loadingAnimFrame >= 6) loadingAnimFrame = 0;
	}
	
	/* Enable/disable throbber */

	var showLoadingIndicator = function(showIt)
	{
		var statusBox = document.getElementById("status");
		if (showIt)
		{
			// Asked to enable - if not already enabled, reset animation frame count and set timer to update frame 
			
			if (!loadingAnim)
			{
				statusBox.addClassName("loading");
				loadingAnimFrame = 0;
				loadingAnim = window.setInterval(updateLoadingIndicator, 120);
			}
		}
		else
		{
			// Asked to disable - if not already off, hide it and disable frame update timer 
			
			statusBox.removeClassName("loading");
			if (loadingAnim) window.clearInterval(loadingAnim);
			loadingAnim = null;
		}
	}
	
	/* Show warning to user that data will be reloaded on drawer close */
	
	var showPendingWarning = function(showIt)
	{
		document.getElementById("reloadpending").style.display = (showIt ? "block" : "none");
	}

	/* Update the label on the bar */

	var setStatusLabel = function(labelString, labelImg)
	{
		var statusLabel = document.getElementById("status_label");
		var statusIcon = document.getElementById("status_icon");
		statusLabel.replaceChild(document.createTextNode(labelString),statusLabel.firstChild);
		if (labelImg) statusIcon.src = labelImg;
	}

/*******************/

	/* Utility to attach action to slider page tabs */

	var attachTabHandlers = function(tabsID, slider)
	{
		var tabs = document.getElementById(tabsID).getElementsByTagName("button");
		
		// For each tab... 

		for (var i=0; i<tabs.length; i++)
		{
			// ...add a click action... 

			tabs[i].addEventListener("mousedown", function(evt)
			{	
				// ...find out the number of the tab, and switch to that page 
				// this involves counting because of JS scoping issues 

				for (var j=0; j<tabs.length; j++)
				{
					if (tabs[j] === evt.target)
					{
						// Switch page, using fast transitions if specified by the drawer manager 

						slider.switchPage(j, drawerMgr.fastSwitch.get());
					}
				}
			}, false);
		}
		
	}

	/* Attach onchange handlers to the config form controls, to update settings when input values are changed */
	
	var attachConfigChangeHandlers = function()
	{
		document.getElementById("location_form").addEventListener("submit", searchLocations, false);
		
		document.getElementById("location_input").addEventListener("focus", function(evt)
		{
			if (evt.target.value == evt.target.defaultValue) evt.target.value = "";		}, false);
		
		// All these change handlers update the associated Preference objects, which may trigger onChange methods to actually
		// activate the changes
		
		// User has selected a new location, tell the weather feed 
		// Weather feed will remember that a data refresh is required 
		
		document.getElementById("location_select").addEventListener("change", function (evt)
		{
			// There is an empty value in the select by default, due to XHTML requirements 
			if (evt.target.value != "")
			{
				weatherFeed.locationCode.set(evt.target.value);
				
				showPendingWarning(true);
			}
		}, false);
		
		// By popular demand - even though we warn that closing the drawer activates changes, reload the data on a double-click too 
		
		document.getElementById("location_select").addEventListener("dblclick", function (evt)
		{
			reloadData();
		}, false);
		
		// The inverse of configDraw.redraw() above - if the radio buttons change, update the settings to match 
		
		var unitsInputs = document.getElementsByName("units");
		var speedRadios = document.getElementsByName("units_speed");
		var tempRadios = document.getElementsByName("temperature");
		
		// Set the new units preferences - again the weatherFeed object remembers whether or not to refresh itself 
		
		for (var i=0; i<unitsInputs.length; i++)
		{
			unitsInputs[i].addEventListener("change", function (evt)
			{
				weatherFeed.basicUnits.set(evt.target.value);
				for (var i=0; i<speedRadios.length; i++)
					speedRadios[i].disabled = (evt.target.value != "metric");
				
				showPendingWarning(true);
			}, false);
		}
		
		for (var i=0; i<speedRadios.length; i++)
		{
			speedRadios[i].addEventListener("change", function (evt)
			{
				weatherFeed.windSpeedKmH.set(evt.target.value == "km/h");
				
				showPendingWarning(true);
			}, false);
		}
		
		for (var i=0; i<tempRadios.length; i++)
		{
			tempRadios[i].addEventListener("change", function (evt)
			{
				weatherFeed.realFeel.set(evt.target.value == "realfeel");
				
				showPendingWarning(true);
			}, false);
		}
		
		// opacity disabled due to incorrect handling of opacity with translucency in current versions of Opera on Windows (perhaps others) 
		//document.getElementById("opacity").addEventListener("change", function(evt)
		//{
		//	opacity.set(evt.target.value);
		//}, false);
		
		document.getElementById("skin").addEventListener("change", function(evt)
		{
			skin.set(evt.target.value);
		}, false);
		
		document.getElementById("fast").addEventListener("change", function(evt)
		{
			drawerMgr.fastSwitch.set(evt.target.checked);
		}, false);
		
		document.getElementById("update_check").addEventListener("change", function(evt)
		{
			updater.enabled.set(evt.target.checked);
		}, false);
	}
	
	
	
}

/*******************/


